@@ -25,7 +25,6 @@ from typing import (
 
 import attr
 import bitmath
-
 from charmonium.determ_hash import determ_hash
 from charmonium.freeze import freeze
 
@@ -50,7 +49,9 @@ BYTE_ORDER: str = "big"
 
 
 def memoize(
-    *, disable: Union[str, bool] = None, **kwargs: Any,
+    *,
+    disable: Union[str, bool] = None,
+    **kwargs: Any,
 ) -> Callable[[Callable[FuncParams, FuncReturn]], Memoized[FuncParams, FuncReturn]]:
     """See :py:class:`charmonium.cache.Memoized`."""
 
@@ -232,7 +233,11 @@ class MemoizedGroup:
         if ops_logger.isEnabledFor(logging.DEBUG):
             ops_logger.debug(
                 json.dumps(
-                    {"event": "cascading_delete", "key": key, "obj_key": obj_key,}
+                    {
+                        "event": "cascading_delete",
+                        "key": key,
+                        "obj_key": obj_key,
+                    }
                 )
             )
 
@@ -287,7 +292,12 @@ class MemoizedGroup:
             )
         if ops_logger.isEnabledFor(logging.DEBUG):
             ops_logger.debug(
-                json.dumps({"event": "index_write", "self._version": self._version,})
+                json.dumps(
+                    {
+                        "event": "index_write",
+                        "self._version": self._version,
+                    }
+                )
             )
 
     def _system_state(self) -> Any:
@@ -356,7 +366,12 @@ class MemoizedGroup:
                 if obj_key not in found_obj_keys:
                     if ops_logger.isEnabledFor(logging.DEBUG):
                         ops_logger.debug(
-                            json.dumps({"event": "remove_orphan", "obj_key": obj_key,})
+                            json.dumps(
+                                {
+                                    "event": "remove_orphan",
+                                    "obj_key": obj_key,
+                                }
+                            )
                         )
                     del self._obj_store[obj_key]
 
@@ -366,6 +381,10 @@ DEFAULT_MEMOIZED_GROUP = Future[MemoizedGroup].create(
 )
 
 
+class CacheThrashingWarning(UserWarning):
+    pass
+
+
 # pyright thinks attrs has ambiguous overload
 @attr.define(init=False, slots=False)  # type: ignore
 class Memoized(Generic[FuncParams, FuncReturn]):
@@ -441,8 +460,7 @@ class Memoized(Generic[FuncParams, FuncReturn]):
             tc = self.group.time_cost[self.name]
             ts = self.group.time_saved[self.name]
         print(
-            "Caching %r: cost %.1fs, saved %.1fs, net saved %.1fs"
-            % (
+            "Caching {}: cost {:.1f}s, saved {:.1f}s, net saved {:.1f}s".format(
                 self.name,
                 tc.total_seconds(),
                 ts.total_seconds(),
@@ -585,7 +603,7 @@ class Memoized(Generic[FuncParams, FuncReturn]):
     ) -> FuncReturn:
         call_start = datetime.datetime.now()
 
-        call_id = random.randint(0, 2 ** 64 - 1)
+        call_id = random.randint(0, 2**64 - 1)
 
         would_hit, key, obj_key = self._would_hit(call_id, *args, **kwargs)
 
@@ -670,13 +688,15 @@ class Memoized(Generic[FuncParams, FuncReturn]):
                         "call_id": call_id,
                         "hit": would_hit,
                         "duration": (call_stop - call_start).total_seconds(),
+                        "obj_key": obj_key,
                     }
                 )
             )
+
         if ts < tc and tc.total_seconds() > 5:
             warnings.warn(
-                f"Caching {self._func} cost {tc.total_seconds():.1f}s but only saved {ts.total_seconds():.1f}s",
-                UserWarning,
+                f"Caching {self._func.__qualname__} cost {tc.total_seconds():.1f}s but only saved {ts.total_seconds():.1f}s",
+                CacheThrashingWarning,
             )
 
         return value
@@ -688,9 +708,9 @@ class Memoized(Generic[FuncParams, FuncReturn]):
             return obj
 
     def would_hit(
-        self, call_id: int, *args: FuncParams.args, **kwargs: FuncParams.kwargs
+        self, *args: FuncParams.args, **kwargs: FuncParams.kwargs
     ) -> bool:
-        return self._would_hit(call_id, *args, **kwargs)[0]
+        return self._would_hit(0, *args, **kwargs)[0]
 
     def _would_hit(
         self, call_id: int, *args: FuncParams.args, **kwargs: FuncParams.kwargs
